module Medidata::API
  require_relative "type"

  # Upload: is generally used for sending payloads in XML format, i.e. sending the invoice.
  class Upload < Dry::Struct
    # Reference to the document sent
    attribute :transmissionReference, Types::Strict::String
    # Date document was delivered
    attribute :created, Types::JSON::DateTime.optional.default(nil)
    # Date of last modification
    attribute :modified, Types::JSON::DateTime.optional.default(nil)
    # Current status of main delivery.
    # The status is a summary of all subdeliveries.
    attribute :status, Types::Strict::String.optional.default("".freeze)
  end

  class UploadControlData < Dry::Struct
    # Transmission mode. The “Mode” control parameter must be identical to
    # the “Mode” attribute in the XML document. Otherwise, an error will be generated.
    attribute :mode, Types::Strict::String.optional.default(nil)
    # Dispatch to organisation (no data warehouse).
    # Overrides the “to” attribute (GLN recipient) in the XML document.
    #
    # In the case of “2000000000008” or “0000000000000”, the following applies
    # • A TP invoice is not delivered to an organisation
    # • A TG invoice is not delivered to an organisation, but to the TG warehouse.
    #
    # An error will be generated with all other differences as compared with
    # the “to” attribute in XML.
    attribute :toOrganization, Types::Strict::String.optional.default(nil)
    # Dispatch to data warehouse (TrustCenter).
    # Overrides the configured data warehouse in the MediData client.
    # This must be a TrustCenter that the sender knows. See participant directory.
    # No dispatch to a data warehouse will be overridden with
    # value “2000000000008” or “0000000000000”.
    attribute :toTrustCenter, Types::Strict::String.optional.default(nil)
    # Dispatch to patients or to their legal representatives (guarantor, debtor).
    #
    # If the value is set to “true”, the dispatch will be triggered; if the
    #  value is set to “false”, dispatch will not take place.
    # In the case of a Tiers Payant (TP) doc- ument, this value overrides the
    #  TP patient copy to the guarantors.
    # In the case of a Tiers Garant (TG) doc- ument, this value overrides the TG
    #  patient invoice and the reimbursement receipt.
    attribute :toPatient, Types::Strict::Bool.default(false)
    # Defines the print language of the document (print layout).
    # If this parameter is not set, it is read from the document or configuration.
    attribute :printLanguage, Types::Strict::String.optional.default(nil)
    # Type of postal dispatch.
    # If this parameter is not set, it is read from the document or configuration.
    attribute :postalDelivery, Types::Strict::String.optional.default(nil)
    # Forces postal delivery to the patient.
    # “true” forces printing with postal delivery
    #  (if multiple types of delivery are possible, such as electronic and post- al).
    # “False” means “no forcing”.
    #  Application: e.g. a reminder should be sent to the guarantor/debtor by post.
    attribute :forcePostalToPatient, Types::Strict::Bool.default(false)
    # Contains a document designation unique to the participant.
    # The reference data must consist solely of alphanumeric characters
    # (incl. special characters) and be between min. 1 and max. 100 characters long.
    attribute :documentReference, Types::Strict::String.optional.default(nil)
    # Contains a business case number. This number is for all sent and received
    # documents that belong to the same business case in technical terms.
    #
    # In this way, all documents belonging to the same business case can be
    # clearly summarised. If this is missing, it is generated from the document
    # content by the MediData client according to the description in the annex.
    #
    # The business number case must consist solely of alphanumeric characters
    # (incl. special characters) and be between min.
    # 1 and max. 100 characters long.
    attribute :correlationReference, Types::Strict::String.optional.default(nil)

    def to_json
      s = {}
      s[:mode] = mode if mode
      s[:toOrganization] = toOrganization if toOrganization
      s[:toTrustCenter] = toTrustCenter if toTrustCenter
      s[:toPatient] = toPatient if toPatient
      s[:printLanguage] = printLanguage if printLanguage
      s[:postalDelivery] = postalDelivery if postalDelivery
      s[:forcePostalToPatient] = forcePostalToPatient if forcePostalToPatient
      s[:documentReference] = documentReference if documentReference
      s[:correlationReference] = correlationReference if correlationReference
      JSON.generate(s)
    end
  end
end